package de.ozgcloud.archive;

import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;

import de.ozgcloud.apilib.client_attribute.OzgCloudClientAttributeService;
import de.ozgcloud.apilib.client_attribute.grpc.OzgCloudClientAttributeGrpcService;
import de.ozgcloud.apilib.client_attribute.grpc.OzgCloudClientAttributeMapper;
import de.ozgcloud.apilib.common.command.OzgCloudCommandService;
import de.ozgcloud.apilib.common.command.grpc.CommandMapper;
import de.ozgcloud.apilib.common.command.grpc.GrpcOzgCloudCommandService;
import de.ozgcloud.apilib.file.OzgCloudFileService;
import de.ozgcloud.apilib.file.grpc.GrpcOzgCloudFileService;
import de.ozgcloud.apilib.file.grpc.OzgCloudFileMapper;
import de.ozgcloud.apilib.user.GrpcOzgCloudUserProfileService;
import de.ozgcloud.apilib.user.OzgCloudUserProfileService;
import de.ozgcloud.apilib.user.UserProfileMapper;
import de.ozgcloud.archive.common.callcontext.CallContextProvider;
import de.ozgcloud.user.grpc.userprofile.UserProfileServiceGrpc.UserProfileServiceBlockingStub;
import de.ozgcloud.vorgang.grpc.binaryFile.BinaryFileServiceGrpc.BinaryFileServiceBlockingStub;
import de.ozgcloud.vorgang.grpc.binaryFile.BinaryFileServiceGrpc.BinaryFileServiceStub;
import de.ozgcloud.vorgang.grpc.clientAttribute.ClientAttributeServiceGrpc.ClientAttributeServiceBlockingStub;
import de.ozgcloud.vorgang.grpc.command.CommandServiceGrpc.CommandServiceBlockingStub;
import lombok.RequiredArgsConstructor;
import net.devh.boot.grpc.client.inject.GrpcClient;

@Configuration
@RequiredArgsConstructor
public class ArchiveManagerConfiguration {

	public static final String VORGANG_SERVICE_NAME = "archive_VorgangService";
	public static final String VORGANG_REMOTE_MANAGER_SERVICE_NAME = "archive_VorgangRemoteService";
	public static final String VORGANG_WITH_EINGANG_MAPPER_NAME = "archive_VorgangWithEingangMapper";

	public static final String CURRENT_USER_SERVICE_NAME = "archive_CurrentUserService";
	public static final String DOCUMENT_MAPPER_NAME = "archive_DocumentMapper";
	public static final String DOCUMENT_REMOTE_SERVICE_NAME = "archive_DocumentRemoteService";

	public static final String USER_SERVICE_NAME = "archive_UserService";
	public static final String USER_PROFILE_MAPPER_NAME = "archive_UserProfileMapper";
	public static final String OZGCLOUD_USER_PROFILE_SERVICE_NAME = "archive_OzgCloudUserProfileService";

	public static final String COMMAND_SERVICE_NAME = "archive_CommandService";
	public static final String COMMAND_REMOTE_SERVICE_NAME = "archive_CommandRemoteService";
	public static final String COMMAND_MAPPER_NAME = "archive_CommandMapper";
	public static final String OZGCLOUD_COMMAND_SERVICE_NAME = "archive_OzgCloudCommandService";

	public static final String BINARY_FILE_SERVICE_NAME = "archive_BinaryFileService";
	public static final String BINARY_FILE_REMOTE_SERVICE_NAME = "archive_BinaryFileRemoteService";
	public static final String OZGCLOUD_FILE_SERVICE_NAME = "archive_OzgCloudFileService";

	public static final String FILE_ID_MAPPER_NAME = "archive_FileIdMapper";

	public static final String BESCHEID_REMOTE_SERVICE_NAME = "archive_BescheidRemoteService";
	public static final String BESCHEID_MAPPER_NAME = "archive_BescheidMapper";

	public static final String CALL_CONTEXT_PROVIDER_NAME = "archive_CallContextProvider";
	public static final String CALL_CONTEXT_MAPPER_NAME = "archive_CallContextMapper";

	public static final String CLIENT_ATTRIBUTE_SERVICE_NAME = "archive_ClientAttributeService";
	public static final String OZGCLOUD_CLIENT_ATTRIBUTE_SERVICE_NAME = "archive_OzgCloudClientAttributeService";

	public static final String KOMMENTAR_SERVICE_NAME = "archive_KommentarService";
	public static final String KOMMENTAR_REMOTE_SERVICE_NAME = "archive_KommentarRemoteService";

	public static final String CALL_CONTEXT_SERVER_INTERCEPTOR_NAME = "archive_CallContextGrpcServerInterceptor";

	public static final String GRPC_VORGANG_MANAGER_CLIENT_NAME = "vorgang-manager";
	public static final String GRPC_COMMAND_MANAGER_CLIENT_NAME = "command-manager";
	public static final String GRPC_FILE_MANAGER_CLIENT_NAME = "file-manager";
	public static final String GRPC_USER_MANAGER_CLIENT_NAME = "user-manager";

	@GrpcClient(GRPC_VORGANG_MANAGER_CLIENT_NAME)
	private final ClientAttributeServiceBlockingStub clientAttributeServiceBlockingStub;

	@GrpcClient(GRPC_COMMAND_MANAGER_CLIENT_NAME)
	private final CommandServiceBlockingStub commandServiceStub;

	@GrpcClient(GRPC_USER_MANAGER_CLIENT_NAME)
	private final UserProfileServiceBlockingStub userProfileServiceBlockingStub;

	@GrpcClient(GRPC_FILE_MANAGER_CLIENT_NAME)
	private final BinaryFileServiceBlockingStub binaryFileServiceBlockingStub;

	@GrpcClient(GRPC_FILE_MANAGER_CLIENT_NAME)
	private final BinaryFileServiceStub binaryFileServiceStub;

	@Bean(ArchiveManagerConfiguration.OZGCLOUD_FILE_SERVICE_NAME) // NOSONAR
	OzgCloudFileService ozgCloudFileService(OzgCloudFileMapper mapper,
			@Qualifier(CALL_CONTEXT_PROVIDER_NAME) CallContextProvider callContextProvider) {
		return new GrpcOzgCloudFileService(binaryFileServiceBlockingStub, binaryFileServiceStub, callContextProvider, mapper);
	}

	@Bean(ArchiveManagerConfiguration.OZGCLOUD_USER_PROFILE_SERVICE_NAME) // NOSONAR
	OzgCloudUserProfileService grpcOzgCloudUserProfileService(UserProfileMapper userProfileMapper,
			@Qualifier(CALL_CONTEXT_PROVIDER_NAME) CallContextProvider callContextProvider) {
		return new GrpcOzgCloudUserProfileService(userProfileServiceBlockingStub, userProfileMapper, callContextProvider);
	}

	@Bean(OZGCLOUD_CLIENT_ATTRIBUTE_SERVICE_NAME) // NOSONAR
	OzgCloudClientAttributeService ozgCloudAttributesService(OzgCloudClientAttributeMapper mapper,
			@Qualifier(CALL_CONTEXT_PROVIDER_NAME) CallContextProvider callContextProvider) {
		return new OzgCloudClientAttributeGrpcService(clientAttributeServiceBlockingStub, mapper, callContextProvider);
	}

	@Bean(OZGCLOUD_COMMAND_SERVICE_NAME) // NOSONAR
	OzgCloudCommandService ozgCloudCommandService(CommandMapper commandMapper,
			@Qualifier(CALL_CONTEXT_PROVIDER_NAME) CallContextProvider callContextProvider) {
		return new GrpcOzgCloudCommandService(commandServiceStub, commandMapper, callContextProvider,
				GrpcOzgCloudCommandService.DEFAULT_COMMAND_REQUEST_THRESHOLD_MILLIS);
	}
}
