/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.archive.attributes;

import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Service;

import de.ozgcloud.apilib.client_attribute.OzgCloudAccessPermission;
import de.ozgcloud.apilib.client_attribute.OzgCloudBooleanClientAttribute;
import de.ozgcloud.apilib.client_attribute.OzgCloudClientAttributeService;
import de.ozgcloud.apilib.vorgang.OzgCloudVorgangIdMapper;
import de.ozgcloud.archive.ArchiveManagerConfiguration;
import de.ozgcloud.archive.common.callcontext.CallContextUser;
import lombok.RequiredArgsConstructor;

@Service(ArchiveManagerConfiguration.CLIENT_ATTRIBUTE_SERVICE_NAME) // NOSONAR
@RequiredArgsConstructor
public class ClientAttributeService {

	public static final String ATTRIBUTE_NAME_ANTRAG_ARCHIVING = "ARCHIVING";

	@Qualifier(ArchiveManagerConfiguration.OZGCLOUD_CLIENT_ATTRIBUTE_SERVICE_NAME) // NOSONAR
	private final OzgCloudClientAttributeService ozgCloudAttributesService;

	private final OzgCloudVorgangIdMapper vorgangIdMapper;

	public void setVorgangArchiving(String vorgangId) {
		ozgCloudAttributesService.setClientAttribute(buildBooleanClientAttribute(true), vorgangIdMapper.fromString(vorgangId));
	}

	OzgCloudBooleanClientAttribute buildBooleanClientAttribute(boolean isArchiving) {
		return OzgCloudBooleanClientAttribute.builder()
				.clientName(CallContextUser.ARCHIVE_MANAGER_CLIENT_NAME)
				.access(OzgCloudAccessPermission.READ_ONLY)
				.attributeName(ATTRIBUTE_NAME_ANTRAG_ARCHIVING)
				.value(isArchiving)
				.build();
	}

}
