package de.ozgcloud.archive.bescheid;

import java.util.stream.Stream;

import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Service;

import de.ozgcloud.archive.ArchiveManagerConfiguration;
import de.ozgcloud.archive.common.binaryfile.BinaryFileService;
import de.ozgcloud.archive.common.user.UserService;
import de.ozgcloud.archive.file.OzgFile;
import de.ozgcloud.archive.vorgang.VorgangWithEingang;
import de.xoev.xdomea.DokumentType;
import lombok.RequiredArgsConstructor;

@Service
@RequiredArgsConstructor
public class ExportBescheidService {

	@Qualifier(ArchiveManagerConfiguration.BESCHEID_REMOTE_SERVICE_NAME) // NOSONAR
	private final BescheidRemoteService bescheidRemoteService;

	@Qualifier(ArchiveManagerConfiguration.BINARY_FILE_SERVICE_NAME) // NOSONAR
	private final BinaryFileService binaryFileService;

	@Qualifier(ArchiveManagerConfiguration.DOCUMENT_REMOTE_SERVICE_NAME) // NOSONAR
	private final DocumentRemoteService documentRemoteService;

	@Qualifier(ArchiveManagerConfiguration.USER_SERVICE_NAME) // NOSONAR
	private final UserService userService;

	public BescheidExportData createExportData(VorgangWithEingang vorgang) {
		var builder = BescheidExportData.builder();
		getSentBescheids(vorgang.getId())
				.forEach(bescheid -> addBescheidExportData(createBescheidExportInput(bescheid, vorgang), builder));
		return builder.build();
	}

	Stream<Bescheid> getSentBescheids(String vorgangId) {
		return bescheidRemoteService.findByVorgangId(vorgangId).filter(bescheid -> BescheidStatus.SENT == bescheid.getStatus());
	}

	BescheidExportInput createBescheidExportInput(Bescheid bescheid, VorgangWithEingang vorgang) {
		return BescheidExportInput.builder()
				.bescheid(bescheid)
				.organisationseinheitenId(vorgang.getOrganisationseinheitenID())
				.files(Stream.concat(Stream.of(getDocument(bescheid)), getAttachments(bescheid)).toList()).build();
	}

	Stream<OzgFile> getAttachments(Bescheid bescheid) {
		return binaryFileService.getFiles(bescheid.getAttachments());
	}

	void addBescheidExportData(BescheidExportInput input, BescheidExportData.BescheidExportDataBuilder builder) {
		builder.dokumentType(buildDokumentType(input))
				.files(input.files());
	}

	OzgFile getDocument(Bescheid bescheid) {
		var documentFileId = documentRemoteService.getDocument(bescheid.getBescheidDocument());
		return binaryFileService.getFile(documentFileId);
	}

	DokumentType buildDokumentType(BescheidExportInput input) {
		var builder = DokumentTypeBuilder.builder();
		builder.withBescheid(input.bescheid())
				.withFiles(input.files())
				.withFullName(getFullName(input.bescheid()))
				.withOrganisationseinheitenId(input.organisationseinheitenId());
		return builder.build();
	}

	String getFullName(Bescheid bescheid) {
		return userService.getFullNameById(bescheid.getSentInfo().getSentBy());
	}

}
