package de.ozgcloud.archive.common;

import java.time.ZonedDateTime;

import org.apache.commons.io.FilenameUtils;
import org.apache.commons.lang3.StringUtils;

import de.ozgcloud.archive.file.OzgFile;
import de.xoev.xdomea.DateiformatCodeType;
import de.xoev.xdomea.FormatType;
import de.xoev.xdomea.VersionType;

public class VersionTypeBuilder {
	public static final String VERSION_NUMMER = "1";
	public static final String DATEI_FORMAT_LIST_URI = "urn:xoev-de:xdomea:codeliste:dateiformat";
	public static final String LIST_VERSION_ID = "2.0";
	private OzgFile ozgFile;
	private ZonedDateTime createdAt;
	private String ersteller;
	private String sonstigerName;

	public static VersionTypeBuilder builder() {
		return new VersionTypeBuilder();
	}

	public VersionTypeBuilder withOzgFile(OzgFile ozgFile) {
		this.ozgFile = ozgFile;
		return this;
	}

	public VersionTypeBuilder withCreatedAt(ZonedDateTime createdAt) {
		this.createdAt = createdAt;
		return this;
	}

	public VersionTypeBuilder withErsteller(String ersteller) {
		this.ersteller = ersteller;
		return this;
	}

	public VersionTypeBuilder withSonstigerName(String sonstigerName) {
		this.sonstigerName = sonstigerName;
		return this;
	}

	public VersionType build() {
		var versionType = new VersionType();
		versionType.setNummer(VERSION_NUMMER);
		versionType.getFormat().add(createFormatType());
		return versionType;
	}

	FormatType createFormatType() {
		var formatType = new FormatType();
		formatType.setName(createDateiformatCodeType());
		formatType.setVersion(StringUtils.EMPTY);
		formatType.setSonstigerName(sonstigerName);
		formatType.setPrimaerdokument(PrimaerdokumentTypeBuilder.builder()
				.withCreatedAt(createdAt)
				.withErsteller(ersteller)
				.withOzgFile(ozgFile)
				.build());
		return formatType;
	}

	DateiformatCodeType createDateiformatCodeType() {
		var dateiformatCode = new DateiformatCodeType();
		dateiformatCode.setCode(DateiformatCode.getXdomeaCode(ozgFile.getContentType(), FilenameUtils.getExtension(ozgFile.getName()).toLowerCase()));
		dateiformatCode.setListURI(DATEI_FORMAT_LIST_URI);
		dateiformatCode.setListVersionID(LIST_VERSION_ID);
		return dateiformatCode;
	}
}
