/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.archive.common.callcontext;

import org.mapstruct.AnnotateWith;
import org.mapstruct.CollectionMappingStrategy;
import org.mapstruct.Mapper;
import org.mapstruct.Mapping;
import org.springframework.stereotype.Component;

import de.ozgcloud.apilib.common.callcontext.CallContext;
import de.ozgcloud.archive.ArchiveManagerConfiguration;
import de.ozgcloud.archive.common.user.OzgCloudUserIdMapper;
import de.ozgcloud.common.grpc.GrpcUtil;
import io.grpc.Metadata;
import lombok.extern.log4j.Log4j2;

@Mapper(uses = OzgCloudUserIdMapper.class, collectionMappingStrategy = CollectionMappingStrategy.ADDER_PREFERRED)
@AnnotateWith(value = Component.class, elements = @AnnotateWith.Element(strings = ArchiveManagerConfiguration.CALL_CONTEXT_MAPPER_NAME))
@Log4j2
abstract class CallContextMapper { // NOSONAR

	@Mapping(target = "accessLimitedToOrgaId", ignore = true)
	@Mapping(target = "clientName", constant = CallContextUser.ARCHIVE_MANAGER_CLIENT_NAME)
	abstract CallContext toOzgCloudCallContext(CallContextUser callContextUser);

	public CallContextUser toCallContextUser(Metadata headers, String requestId) {
		var builder = CallContextUser.builder()
				.requestId(requestId)
				.userId(GrpcUtil.getUserId(headers))
				.userName(GrpcUtil.getUserName(headers));
		GrpcUtil.getAccessLimited(headers).ifPresentOrElse(
				isAccessLimited -> builder.accessLimited(isAccessLimited).accessLimitedToOrgaIds(GrpcUtil.getAccessLimitedOrgaIds(headers)),
				() -> builder.accessLimited(false));
		// TODO throw exception if missing required data as soon all clients are fine
		GrpcUtil.getClientName(headers).ifPresentOrElse(builder::clientName, () -> LOG.warn("Missing client name in grpc header."));
		return builder.build();
	}
}
