package de.ozgcloud.archive.common.command;

import java.time.ZonedDateTime;

import org.apache.commons.lang3.StringUtils;
import org.mapstruct.AnnotateWith;
import org.mapstruct.AnnotateWith.Element;
import org.mapstruct.Mapper;
import org.mapstruct.Mapping;
import org.mapstruct.NullValueCheckStrategy;
import org.springframework.stereotype.Component;

import de.ozgcloud.archive.ArchiveManagerConfiguration;
import de.ozgcloud.vorgang.common.grpc.GrpcObjectMapper;
import de.ozgcloud.vorgang.grpc.command.GrpcCommand;

@Mapper(uses = { GrpcObjectMapper.class }, nullValueCheckStrategy = NullValueCheckStrategy.ALWAYS)
@AnnotateWith(value = Component.class, elements = @Element(strings = ArchiveManagerConfiguration.COMMAND_MAPPER_NAME))
interface CommandMapper {

	@Mapping(target = "body", source = "bodyObj")
	@Mapping(target = "order", source = "orderString")
	@Mapping(target = "finishedAt", expression = "java(mapToZonedDateTime(grpcCommand.getFinishedAt()))")
	ArchiveManagerCommand fromGrpc(GrpcCommand grpcCommand);

	default ZonedDateTime mapToZonedDateTime(String value) {
		return StringUtils.isEmpty(value) ? null : ZonedDateTime.parse(value);
	}
}
