package de.ozgcloud.archive.common.user;

import java.util.Optional;

import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Service;

import de.ozgcloud.apilib.user.OzgCloudUserId;
import de.ozgcloud.apilib.user.OzgCloudUserProfileService;
import de.ozgcloud.archive.ArchiveManagerConfiguration;
import lombok.RequiredArgsConstructor;

@Service(ArchiveManagerConfiguration.USER_SERVICE_NAME) // NOSONAR
@RequiredArgsConstructor
public class UserService {

	private static final String SYSTEM_USER_ID_PREFIX = "system_";

	@Qualifier(ArchiveManagerConfiguration.OZGCLOUD_USER_PROFILE_SERVICE_NAME) // NOSONAR
	private final OzgCloudUserProfileService grpcOzgCloudUserProfileService;

	@Qualifier(ArchiveManagerConfiguration.USER_PROFILE_MAPPER_NAME) // NOSONAR
	private final UserProfileMapper mapper;

	public String getFullNameById(String userId) {
		// TODO (Story offen) es soll der Name des Systems geliefert werden, wenn kein
		// Mensch die Aktion ausgeführt hat
		return getById(userId).map(UserProfile::getFullName).orElse("automatisch");
	}

	public Optional<UserProfile> getById(String userId) {
		return Optional.of(userId).filter(id -> !id.startsWith(SYSTEM_USER_ID_PREFIX))
				.map(OzgCloudUserId::from)
				.map(grpcOzgCloudUserProfileService::getById)
				.map(mapper::fromOzgUserProfile);
	}

}
