package de.ozgcloud.archive.export;

import java.util.HashMap;
import java.util.Map;

import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.oxm.jaxb.Jaxb2Marshaller;
import org.springframework.validation.Validator;

import jakarta.xml.bind.Marshaller;
import lombok.RequiredArgsConstructor;

@RequiredArgsConstructor
@Configuration
class ExportConfiguration {

	static final boolean JAXB_FORMATTED_OUTPUT = true;
	static final String PROPERTY_NAMESPACE_PREFIX_MAPPER = "org.glassfish.jaxb.namespacePrefixMapper";
	static final String CONTEXT_PATH = "de.xoev.xdomea";

	private final XdomeaNamespacePrefixMapper prefixMapper;

	@Bean
	static Validator configurationPropertiesValidator() {
		return new XdomeaPropertiesValidator();
	}

	@Bean
	Jaxb2Marshaller marshaller() {
		var marshaller = new Jaxb2Marshaller();
		marshaller.setContextPaths(CONTEXT_PATH);
		marshaller.setMarshallerProperties(createMarshallerProperties());
		return marshaller;
	}

	Map<String, Object> createMarshallerProperties() {
		var props = new HashMap<String, Object>();
		props.put(Marshaller.JAXB_FORMATTED_OUTPUT, JAXB_FORMATTED_OUTPUT);
		props.put(PROPERTY_NAMESPACE_PREFIX_MAPPER, prefixMapper);
		return props;
	}
}
