package de.ozgcloud.archive.export;

import java.util.UUID;

import org.springframework.core.task.TaskExecutor;

import com.google.protobuf.ByteString;

import de.ozgcloud.archive.ArchiveManagerConfiguration;
import de.ozgcloud.archive.grpc.export.ExportServiceGrpc.ExportServiceImplBase;
import de.ozgcloud.archive.grpc.export.GrpcExportVorgangRequest;
import de.ozgcloud.archive.grpc.export.GrpcExportVorgangResponse;
import de.ozgcloud.archive.grpc.export.GrpcFile;
import de.ozgcloud.common.binaryfile.GrpcBinaryFileServerDownloader;
import io.grpc.stub.CallStreamObserver;
import io.grpc.stub.StreamObserver;
import lombok.RequiredArgsConstructor;
import net.devh.boot.grpc.server.service.GrpcService;

@GrpcService(interceptorNames = { ArchiveManagerConfiguration.CALL_CONTEXT_SERVER_INTERCEPTOR_NAME })
@RequiredArgsConstructor
class ExportGrpcService extends ExportServiceImplBase {

	private static final String EXPORT_FILENAME_TEMPLATE = "%s_Abgabe.Abgabe.0401.xdomea";

	static final int CHUNK_SIZE = 256 * 1024;

	private final ExportService exportService;
	private final TaskExecutor taskExecutor;

	@Override
	public void exportVorgang(GrpcExportVorgangRequest request, StreamObserver<GrpcExportVorgangResponse> responseObserver) {
		var fileNameId = createFileNameId();
		sendFileName(responseObserver, fileNameId);
		buildExportDownloader(request.getVorgangId(), fileNameId, responseObserver).start();
	}

	String createFileNameId() {
		return UUID.randomUUID().toString();
	}

	void sendFileName(StreamObserver<GrpcExportVorgangResponse> responseObserver, String fileNameId) {
		responseObserver.onNext(GrpcExportVorgangResponse.newBuilder()
				.setVorgangFile(GrpcFile.newBuilder().setFileName(buildXdomeaFileName(fileNameId)).build())
				.build());
	}

	String buildXdomeaFileName(String fileNameId) {
		return String.format(EXPORT_FILENAME_TEMPLATE, fileNameId);
	}

	GrpcBinaryFileServerDownloader<GrpcExportVorgangResponse> buildExportDownloader(String vorgangId, String fileNameId,
			StreamObserver<GrpcExportVorgangResponse> responseObserver) {
		return GrpcBinaryFileServerDownloader.<GrpcExportVorgangResponse>builder()
				.callObserver((CallStreamObserver<GrpcExportVorgangResponse>) responseObserver)
				.taskExecutor(taskExecutor)
				.downloadConsumer(outputStream -> exportService.writeXdomeaFileContent(vorgangId, fileNameId, outputStream))
				.chunkBuilder(this::buildExportVorgangChunkResponse)
				.build();
	}

	GrpcExportVorgangResponse buildExportVorgangChunkResponse(ByteString chunk) {
		return GrpcExportVorgangResponse.newBuilder()
				.setVorgangFile(GrpcFile.newBuilder().setFileContent(chunk).build())
				.build();
	}
}
