package de.ozgcloud.archive.file;

import java.util.stream.Stream;

import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Service;

import de.ozgcloud.apilib.common.callcontext.OzgCloudCallContextAttachingInterceptor;
import de.ozgcloud.apilib.common.callcontext.OzgCloudCallContextProvider;
import de.ozgcloud.archive.ArchiveManagerConfiguration;
import de.ozgcloud.vorgang.grpc.file.FileServiceGrpc.FileServiceBlockingStub;
import de.ozgcloud.vorgang.grpc.file.GrpcGetAttachmentsRequest;
import de.ozgcloud.vorgang.grpc.file.GrpcGetRepresentationsRequest;
import lombok.RequiredArgsConstructor;
import net.devh.boot.grpc.client.inject.GrpcClient;

@Service
@RequiredArgsConstructor
class OzgFileRemoteService {

	@GrpcClient(ArchiveManagerConfiguration.GRPC_VORGANG_MANAGER_CLIENT_NAME) // NOSONAR
	private final FileServiceBlockingStub grpcService;
	private final OzgFileMapper fileMapper;
	@Qualifier(ArchiveManagerConfiguration.CALL_CONTEXT_PROVIDER_NAME) // NOSONAR
	private final OzgCloudCallContextProvider contextProvider;

	public Stream<OzgFile> getAttachments(String vorgangId) {
		return getGrpcServiceWithInterceptor().getAttachments(buildGrpcGetAttachmentsRequest(vorgangId)).getFileList().stream()
				.map(fileMapper::fromGrpc);
	}

	GrpcGetAttachmentsRequest buildGrpcGetAttachmentsRequest(String vorgangId) {
		return GrpcGetAttachmentsRequest.newBuilder().setVorgangId(vorgangId).build();
	}

	public Stream<OzgFile> getRepresentations(String vorgangId) {
		return getGrpcServiceWithInterceptor().getRepresentations(buildGrpcGetRepresentationsRequest(vorgangId)).getFileList().stream()
				.map(fileMapper::fromGrpc);
	}

	GrpcGetRepresentationsRequest buildGrpcGetRepresentationsRequest(String vorgangId) {
		return GrpcGetRepresentationsRequest.newBuilder().setVorgangId(vorgangId).build();
	}

	FileServiceBlockingStub getGrpcServiceWithInterceptor() {
		return grpcService.withInterceptors(new OzgCloudCallContextAttachingInterceptor(contextProvider));
	}
}
