package de.ozgcloud.archive.historie;

import java.util.Optional;

import org.apache.commons.lang3.StringUtils;

import de.ozgcloud.archive.common.command.ArchiveManagerCommand;
import de.ozgcloud.archive.common.command.CommandWithPrevious;
import lombok.AccessLevel;
import lombok.NoArgsConstructor;

@NoArgsConstructor(access = AccessLevel.PRIVATE)
class AktenzeichenChangeHistoryBuilder extends ChangeHistoryBuilder<AktenzeichenChangeHistoryBuilder> {

	static final String BODY_PROPERTY_AKTENZEICHEN = "aktenzeichen";
	static final String SET_AKTENZEICHEN_ORDER = "SET_AKTENZEICHEN";

	public static AktenzeichenChangeHistoryBuilder builder() {
		return new AktenzeichenChangeHistoryBuilder();
	}

	@Override
	boolean isRelevant(ArchiveManagerCommand command) {
		return SET_AKTENZEICHEN_ORDER.equals(command.getOrder());
	}

	@Override
	CommandWithChangeValues toCommandWithChangeValues(CommandWithPrevious commandWithPrevious) {
		return new CommandWithChangeValues(
				commandWithPrevious.getCommand(),
				getAktenzeichenBeforeChange(commandWithPrevious),
				getAktenzeichenAfterChange(commandWithPrevious));
	}

	String getAktenzeichenBeforeChange(CommandWithPrevious commandWithPrevious) {
		return Optional.ofNullable(commandWithPrevious.getPrevious())
				.map(this::getAktenzeichen)
				.orElse(StringUtils.EMPTY);
	}

	String getAktenzeichenAfterChange(CommandWithPrevious commandWithPrevious) {
		return getAktenzeichen(commandWithPrevious.getCommand());
	}

	String getAktenzeichen(ArchiveManagerCommand command) {
		return getValueFromCommandBody(BODY_PROPERTY_AKTENZEICHEN, command);
	}
}
