package de.ozgcloud.archive.historie;

import java.util.HashMap;
import java.util.Map;
import java.util.Optional;
import java.util.function.Function;

import de.ozgcloud.archive.common.user.UserProfile;
import de.ozgcloud.common.errorhandling.TechnicalException;
import lombok.AccessLevel;
import lombok.RequiredArgsConstructor;

//TODO OZG-7379 remove this cache
@RequiredArgsConstructor(access = AccessLevel.PRIVATE)
class UserProfileCache {

	private final Function<String, Optional<UserProfile>> findUserProfileForUserId;

	private final Map<String, Optional<UserProfile>> cache = new HashMap<>();

	public static UserProfileCache create(Function<String, Optional<UserProfile>> findUserProfileForUserId) {
		return new UserProfileCache(findUserProfileForUserId);
	}

	public Optional<UserProfile> getUserProfile(String userId) {
		if (!cache.containsKey(userId)) {
			findAndAddToCache(userId);
		}
		return cache.get(userId);
	}

	private void findAndAddToCache(String userId) {
		var userProfile = findUserProfileForUserId.apply(userId);
		if (userProfile == null) {
			throw new TechnicalException("UserProfile not found for user with ID " + userId);
		}
		cache.put(userId, userProfile);
	}
}
