/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.archive.kommentar;

import java.util.Map;
import java.util.Objects;

import org.mapstruct.Mapper;
import org.mapstruct.NullValueCheckStrategy;
import org.mapstruct.ReportingPolicy;
import org.springframework.beans.factory.annotation.Autowired;

import de.ozgcloud.archive.common.TimeMapper;
import de.ozgcloud.vorgang.common.GrpcObject;
import de.ozgcloud.vorgang.common.grpc.GrpcObjectMapper;
import de.ozgcloud.vorgang.vorgangAttachedItem.GrpcVorgangAttachedItem;

@Mapper(unmappedTargetPolicy = ReportingPolicy.ERROR, nullValueCheckStrategy = NullValueCheckStrategy.ALWAYS, uses = { TimeMapper.class,
		AttachmentMapper.class })
abstract class KommentarMapper {

	@Autowired
	private GrpcObjectMapper grpcObjectMapper;

	public Kommentar fromItem(GrpcVorgangAttachedItem vorgangAttachedItem) {
		return mapItemToKommentar(vorgangAttachedItem.getItem());
	}

	Kommentar mapItemToKommentar(GrpcObject item) {
		return mapItemMapToKommentar(mapItemToItemMap(item));
	}

	Map<String, Object> mapItemToItemMap(GrpcObject item) {
		return grpcObjectMapper.mapFromGrpc(item);
	}

	abstract Kommentar mapItemMapToKommentar(Map<String, Object> map);

	String mapObjectToString(Object object) {
		return Objects.nonNull(object) ? Objects.toString(object) : null;
	}
}
