package de.ozgcloud.archive.vorgang;

import de.ozgcloud.archive.common.DatentypCode;
import de.xoev.xdomea.DatentypCodeType;
import de.xoev.xdomea.FeldType;

enum ExportFelder {
	LEIKA_ID("LeikaID", "ID einer Leistung aus dem OZG-Leistungskatalog", DatentypCode.STRING.getCode()),
	DATUM_ANTRAGSEINGANG("DatumAntragseingang", "Das Datum des Antragseingangs", DatentypCode.DATE.getCode()),
	NAME("Name", null, DatentypCode.STRING.getCode()),
	VORNAME("Vorname", null, DatentypCode.STRING.getCode()),
	GEBURTSDATUM("Geburtsdatum", null, DatentypCode.DATE.getCode()),
	PLZ("PLZAntragsteller", null, DatentypCode.STRING.getCode());

	private static final String LIST_URI = "urn:xoev-de:xdomea:codeliste:datentyp";
	private static final String LIST_VERSION_ID = "1.1";

	private String name;
	private String beschreibung;
	private String datentypCode;

	ExportFelder(String name, String beschreibung, String datentypCode) {
		this.name = name;
		this.beschreibung = beschreibung;
		this.datentypCode = datentypCode;
	}

	public String getName() {
		return name;
	}

	public String getBeschreibung() {
		return beschreibung;
	}

	public String getDatentypCode() {
		return datentypCode;
	}

	public FeldType createFeld(String wert) {
		var feld = createFeld();
		feld.setWert(wert);
		return feld;
	}

	public FeldType createFeld() {
		var feld = new FeldType();
		feld.setName(name);
		feld.setBeschreibung(beschreibung);
		feld.setDatentyp(fromCode(datentypCode));
		return feld;
	}

	private DatentypCodeType fromCode(String code) {
		var type = new DatentypCodeType();
		type.setCode(code);
		type.setListURI(LIST_URI);
		type.setListVersionID(LIST_VERSION_ID);
		return type;
	}
}
