package de.ozgcloud.archive.vorgang;

import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Service;

import de.ozgcloud.apilib.common.callcontext.OzgCloudCallContextAttachingInterceptor;
import de.ozgcloud.apilib.common.callcontext.OzgCloudCallContextProvider;
import de.ozgcloud.archive.ArchiveManagerConfiguration;
import de.ozgcloud.vorgang.vorgang.GrpcFindVorgangWithEingangRequest;
import de.ozgcloud.vorgang.vorgang.VorgangServiceGrpc.VorgangServiceBlockingStub;
import lombok.RequiredArgsConstructor;
import net.devh.boot.grpc.client.inject.GrpcClient;

@Service(ArchiveManagerConfiguration.VORGANG_REMOTE_MANAGER_SERVICE_NAME) // NOSONAR
@RequiredArgsConstructor
class VorgangRemoteService {

	@GrpcClient(ArchiveManagerConfiguration.GRPC_VORGANG_MANAGER_CLIENT_NAME) // NOSONAR
	private final VorgangServiceBlockingStub grpcService;

	@Qualifier(ArchiveManagerConfiguration.VORGANG_WITH_EINGANG_MAPPER_NAME) // NOSONAR
	private final VorgangWithEingangMapper vorgangWithEingangMapper;

	@Qualifier(ArchiveManagerConfiguration.CALL_CONTEXT_PROVIDER_NAME) // NOSONAR
	private final OzgCloudCallContextProvider contextProvider;

	public VorgangWithEingang findVorgangWithEingang(String vorgangId) {
		return vorgangWithEingangMapper
				.fromGrpc(getGrpcServiceWithInterceptor()
						.findVorgangWithEingang(buildFindVorgangWithEingangRequest(vorgangId))
						.getVorgangWithEingang());
	}

	GrpcFindVorgangWithEingangRequest buildFindVorgangWithEingangRequest(String id) {
		return GrpcFindVorgangWithEingangRequest.newBuilder()
				.setId(id)
				.build();
	}

	VorgangServiceBlockingStub getGrpcServiceWithInterceptor() {
		return grpcService.withInterceptors(new OzgCloudCallContextAttachingInterceptor(contextProvider));
	}
}
