package de.ozgcloud.archive;

import java.util.TimeZone;

import org.mapstruct.factory.Mappers;
import org.springframework.boot.SpringApplication;
import org.springframework.boot.autoconfigure.SpringBootApplication;
import org.springframework.boot.info.BuildProperties;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.ComponentScan;
import org.springframework.security.authentication.AuthenticationTrustResolver;
import org.springframework.security.authentication.AuthenticationTrustResolverImpl;

import de.ozgcloud.apilib.client_attribute.grpc.OzgCloudClientAttributeMapper;
import de.ozgcloud.apilib.common.command.grpc.CommandMapper;
import de.ozgcloud.apilib.file.grpc.OzgCloudFileMapper;
import de.ozgcloud.apilib.user.UserProfileMapper;
import de.ozgcloud.apilib.vorgang.OzgCloudVorgangIdMapper;
import de.ozgcloud.apilib.vorgang.grpc.OzgCloudVorgangMapper;
import de.ozgcloud.vorgang.vorgang.VorgangServiceGrpc;

@SpringBootApplication
@ComponentScan({ "de.ozgcloud.*" })
public class ArchiveManagerTestApplication {

	@MockBean
	private OzgCloudVorgangMapper vorgangMapper;
	@MockBean
	private VorgangServiceGrpc.VorgangServiceBlockingStub vorgangServiceStub;
	@MockBean
	private OzgCloudVorgangIdMapper vorgangIdMapper;
	@MockBean
	private UserProfileMapper userProfileMapper;
	@MockBean
	private OzgCloudFileMapper fileMapper;
	@MockBean
	private BuildProperties properties;
	@MockBean
	private OzgCloudClientAttributeMapper clientAttributeMapper;

	public static void main(String[] args) {
		TimeZone.setDefault(TimeZone.getTimeZone("UTC"));
		SpringApplication.run(ArchiveManagerTestApplication.class, args);
	}

	@Bean
	AuthenticationTrustResolver trustResolver() {
		return new AuthenticationTrustResolverImpl();
	}

	@Bean
	CommandMapper commandMapper() {
		return Mappers.getMapper(CommandMapper.class);
	}

}
