package de.ozgcloud.archive.common;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import java.util.Random;

import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.MockedStatic;
import org.mockito.Spy;

import de.ozgcloud.archive.export.IdentifikationObjektTypeTestFactory;
import de.ozgcloud.archive.file.OzgFileTestFactory;

class IdentifikationObjektTypeBuilderTest {
	private static final Long DOKUMENT_ORDINAL_NUMBER = new Random().nextLong();

	@Spy
	private IdentifikationObjektTypeBuilder builder = IdentifikationObjektTypeBuilder.builder().withObjectID(OzgFileTestFactory.ID.toString())
			.withOrdinalNumber(DOKUMENT_ORDINAL_NUMBER);

	@Nested
	class TestBuild {
		private MockedStatic<UUIDConverter> uuidConverterMockedStatic;

		@BeforeEach
		void setUp() {
			uuidConverterMockedStatic = mockStatic(UUIDConverter.class);
			uuidConverterMockedStatic.when(() -> UUIDConverter.fromObjectId(OzgFileTestFactory.ID.toString()))
					.thenReturn(IdentifikationObjektTypeTestFactory.ID);
		}

		@AfterEach
		void tearDown() {
			uuidConverterMockedStatic.close();
		}

		@Test
		void shouldConvertOzgFileId() {
			builder.build();

			uuidConverterMockedStatic.verify(() -> UUIDConverter.fromObjectId(OzgFileTestFactory.ID.toString()));
		}

		@Test
		void shouldHaveId() {
			var identifikation = builder.build();

			assertThat(identifikation.getID()).isEqualTo(IdentifikationObjektTypeTestFactory.ID);
		}

		@Test
		void shouldHaveNummerImUebergeordnetenContainer() {
			var identifikation = builder.build();

			assertThat(identifikation.getNummerImUebergeordnetenContainer()).isEqualTo(DOKUMENT_ORDINAL_NUMBER);
		}
	}

}
