package de.ozgcloud.archive.common.callcontext;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;

import de.ozgcloud.apilib.common.callcontext.CallContext;

class CallContextProviderTest {

	@Spy
	@InjectMocks
	private CallContextProvider contextProvider;

	@Mock
	private CallContextMapper mapper;
	@Mock
	private CurrentUserService currentUserService;

	@Nested
	class TestProvideContext {

		private final CallContextUser callContextUser = CallContextUserTestFactory.create();

		@BeforeEach
		void mock() {
			when(currentUserService.getUser()).thenReturn(callContextUser);
		}

		@Test
		void shouldGetArchiveCallContextFromGrpcContext() {
			callProvider();

			verify(currentUserService).getUser();
		}

		@Test
		void shouldMapArchiveCallContext() {
			callProvider();

			verify(mapper).toOzgCloudCallContext(callContextUser);
		}

		@Test
		void shouldReturnMappedCallContext() {
			var mappedCallContext = CallContextTestFactory.create();
			when(mapper.toOzgCloudCallContext(callContextUser)).thenReturn(mappedCallContext);

			var context = callProvider();

			assertThat(context).isEqualTo(mappedCallContext);
		}

		private CallContext callProvider() {
			return contextProvider.provideContext();
		}
	}
}
