package de.ozgcloud.archive.common.user;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;

import de.ozgcloud.apilib.user.GrpcOzgCloudUserProfileService;
import de.ozgcloud.apilib.user.OzgCloudUserId;
import de.ozgcloud.apilib.user.OzgCloudUserProfile;
import de.ozgcloud.apilib.user.OzgCloudUserProfileTestFactory;

class UserServiceTest {

	@InjectMocks
	private UserService service;

	@Mock
	private GrpcOzgCloudUserProfileService grpcService;

	@Mock
	private UserProfileMapper mapper;

	@Nested
	class TestGetById {

		private final OzgCloudUserProfile ozgCloudUserProfile = OzgCloudUserProfileTestFactory.create();

		@Test
		void shouldCallUserProfileService() {
			service.getById(UserProfileTestFactory.USER_ID);

			verify(grpcService).getById(OzgCloudUserId.from(UserProfileTestFactory.USER_ID));
		}

		@Test
		void shouldMap() {
			when(grpcService.getById(OzgCloudUserId.from(UserProfileTestFactory.USER_ID))).thenReturn(ozgCloudUserProfile);

			service.getById(UserProfileTestFactory.USER_ID);

			verify(mapper).fromOzgUserProfile(ozgCloudUserProfile);
		}

		@Test
		void shouldRetrunMappedUserProfile() {
			var mappedUserProfile = UserProfileTestFactory.create();
			when(grpcService.getById(any())).thenReturn(ozgCloudUserProfile);
			when(mapper.fromOzgUserProfile(ozgCloudUserProfile)).thenReturn(mappedUserProfile);

			var userProfile = service.getById(UserProfileTestFactory.USER_ID);

			assertThat(userProfile).contains(mappedUserProfile);
		}

		@Test
		void shouldReturnEmptyForSystemId() {
			var result = service.getById("system_test");

			assertThat(result).isEmpty();
		}

		@Test
		void shoultNOTCallRemoteServiceForSystemId() {
			service.getById("system_test");

			verify(grpcService, never()).getById(any());
		}

	}
}
