package de.ozgcloud.archive.export;

import static org.junit.jupiter.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import java.io.ByteArrayOutputStream;
import java.util.Optional;
import java.util.UUID;
import java.util.stream.Stream;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.boot.test.mock.mockito.SpyBean;

import de.ozgcloud.archive.bescheid.BescheidExportDataTestFactory;
import de.ozgcloud.archive.bescheid.ExportBescheidService;
import de.ozgcloud.archive.common.binaryfile.BinaryFileService;
import de.ozgcloud.archive.common.user.UserProfileTestFactory;
import de.ozgcloud.archive.common.user.UserService;
import de.ozgcloud.archive.file.ExportFileService;
import de.ozgcloud.archive.file.OzgFileTestFactory;
import de.ozgcloud.archive.historie.ExportHistorieService;
import de.ozgcloud.archive.kommentar.ExportKommentarService;
import de.ozgcloud.archive.kommentar.KommentarsExportDataTestFactory;
import de.ozgcloud.archive.postfach.ExportNachrichtService;
import de.ozgcloud.archive.postfach.PostfachMailExportDataTestFactory;
import de.ozgcloud.archive.vorgang.VorgangService;
import de.ozgcloud.archive.vorgang.VorgangWithEingang;
import de.ozgcloud.archive.vorgang.VorgangWithEingangTestFactory;
import de.ozgcloud.common.test.ITCase;

@ITCase
class ExportServiceITCase {

	@SpyBean
	private ExportFileService exportFileService;
	@SpyBean
	private VorgangService vorgangService;
	@MockBean
	private ExportHistorieService exportHistorieService;
	@MockBean
	private ExportKommentarService exportKommentarService;
	@MockBean
	private BinaryFileService binaryFileService;
	@MockBean
	private ExportNachrichtService exportNachrichtService;
	@MockBean
	private ExportBescheidService exportBescheidService;
	@MockBean
	private UserService userService;
	@Autowired
	private ExportService exportService;

	@Nested
	class TestWriteExport {

		private final VorgangWithEingang vorgang = VorgangWithEingangTestFactory.create();
		private final String userId = UUID.randomUUID().toString();

		@BeforeEach
		void setup() {
			doReturn(vorgang).when(vorgangService).getVorgang(VorgangWithEingangTestFactory.ID);
			doReturn(Stream.of(OzgFileTestFactory.createWithUniqueId())).when(exportFileService).getRepresentations(any());
			doReturn(Stream.of(OzgFileTestFactory.createWithUniqueId())).when(exportFileService).getAttachments(any());
			doReturn(Stream.of(OzgFileTestFactory.createWithUniqueId())).when(binaryFileService).getFiles(any());
			doReturn(Optional.of(UserProfileTestFactory.create())).when(userService).getById(any());
			doNothing().when(exportFileService).writeOzgFile(any(), any());
			when(exportHistorieService.createHistorienProtokollInformationTypes(vorgang)).thenReturn(Stream.empty());
			when(exportKommentarService.createExportData(vorgang)).thenReturn(KommentarsExportDataTestFactory.create());
			when(exportNachrichtService.createExportData(vorgang)).thenReturn(PostfachMailExportDataTestFactory.create());
			when(exportBescheidService.createExportData(vorgang)).thenReturn(BescheidExportDataTestFactory.create());
		}

		@Test
		void shouldNotThrowException() {
			assertDoesNotThrow(
					() -> exportService.writeXdomeaFileContent(VorgangWithEingangTestFactory.ID, userId, new ByteArrayOutputStream()));
		}
	}
}
