package de.ozgcloud.archive.kommentar;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mapstruct.factory.Mappers;
import org.mockito.InjectMocks;
import org.mockito.Mock;

import de.ozgcloud.archive.common.TimeMapper;
import de.ozgcloud.vorgang.common.grpc.GrpcObjectMapper;

class KommentarMapperITCase {

	@InjectMocks
	private final KommentarMapper kommentarMapper = Mappers.getMapper(KommentarMapper.class);
	@Mock
	private GrpcObjectMapper grpcObjectMapper;
	@Mock
	private TimeMapper timeMapper;
	@Mock
	private AttachmentMapper attachmentMapper;

	@Nested
	class TestFromItem {

		@BeforeEach
		void setUpCallsOfDependentMappers() {
			when(grpcObjectMapper.mapFromGrpc(any())).thenCallRealMethod();
			when(grpcObjectMapper.mapProperty(any())).thenCallRealMethod();
			when(timeMapper.parseString(any())).thenCallRealMethod();
			when(attachmentMapper.mapAttachments(any())).thenCallRealMethod();
		}

		@Test
		void shouldMapId() {
			var kommentar = callMapper();

			assertThat(kommentar.getId()).isEqualTo(KommentarTestFactory.ID);
		}

		@Test
		void shouldMapText() {
			var kommentar = callMapper();

			assertThat(kommentar.getText()).isEqualTo(KommentarTestFactory.TEXT);
		}

		@Test
		void shouldMapCreatedBy() {
			var kommentar = callMapper();

			assertThat(kommentar.getCreatedBy()).isEqualTo(KommentarTestFactory.CREATED_BY);
		}

		@Test
		void shouldMapCreatedAt() {
			var kommentar = callMapper();

			assertThat(kommentar.getCreatedAt()).isEqualTo(KommentarTestFactory.CREATED_AT);
		}

		@Test
		void shouldMapAttachments() {
			var kommentar = callMapper();

			assertThat(kommentar.getAttachments()).containsExactlyElementsOf(KommentarTestFactory.ATTACHMENTS);
		}

		private Kommentar callMapper() {
			return kommentarMapper.fromItem(GrpcKommentarTestFactory.create());
		}
	}
}
